==> bash-language-server-1.7.0-2-aarch64.pkg.tar.xz: usr/lib/bash-language-server/src/executables.ts <==
import * as Fs from 'fs'
import * as Path from 'path'

import * as ArrayUtil from './util/array'
import * as FsUtil from './util/fs'
import * as ShUtil from './util/sh'

/**
 * Provides information based on the programs on your PATH
 */
export default class Executables {
  /**
   * @param path is expected to to be a ':' separated list of paths.
   */
  public static fromPath(path: string): Promise<Executables> {
    const paths = path.split(':')
    const promises = paths.map(x => findExecutablesInPath(x))
    return Promise.all(promises)
      .then(ArrayUtil.flatten)
      .then(ArrayUtil.uniq)
      .then(executables => new Executables(executables))
  }

  private executables: Set<string>

  private constructor(executables: string[]) {
    this.executables = new Set(executables)
  }

  /**
   * Find all programs in your PATH
   */
  public list(): string[] {
    return Array.from(this.executables.values())
  }

  /**
   * Check if the the given {{executable}} exists on the PATH
   */
  public isExecutableOnPATH(executable: string): boolean {
    return this.executables.has(executable)
  }

  /**
   * Look up documentation for the given executable.
   *
   * For now it simply tries to look up the MAN documentation.
   */
  public documentation(executable: string): Promise<string> {
    return ShUtil.execShellScript(`man ${executable} | col -b`).then(doc => {
      return !doc
        ? Promise.resolve(`No MAN page for ${executable}`)
        : Promise.resolve(doc)
    })
  }
}

/**
 * Only returns direct children, or the path itself if it's an executable.
 */
function findExecutablesInPath(path: string): Promise<string[]> {
  path = FsUtil.untildify(path)
  return new Promise((resolve, _) => {
    Fs.lstat(path, (err, stat) => {
      if (err) {
        resolve([])
      } else {
        if (stat.isDirectory()) {
          Fs.readdir(path, (readDirErr, paths) => {
            if (readDirErr) {
              resolve([])
            } else {
              const files = paths.map(p =>
                FsUtil.getStats(Path.join(path, p))
                  .then(s => (s.isFile() ? [Path.basename(p)] : []))
                  .catch(() => []),
              )

              resolve(Promise.all(files).then(ArrayUtil.flatten))
            }
          })
        } else if (stat.isFile()) {
          resolve([Path.basename(path)])
        } else {
          // Something else.
          resolve([])
        }
      }
    })
  })
}

==> bash-language-server-1.13.1-2-aarch64.pkg.tar.xz: usr/lib/bash-language-server/src/executables.ts <==
import * as fs from 'fs'
import { basename, join } from 'path'
import { promisify } from 'util'

import * as ArrayUtil from './util/array'
import * as FsUtil from './util/fs'
import * as ShUtil from './util/sh'

const lstatAsync = promisify(fs.lstat)
const readdirAsync = promisify(fs.readdir)

/**
 * Provides information based on the programs on your PATH
 */
export default class Executables {
  /**
   * @param path is expected to to be a ':' separated list of paths.
   */
  public static fromPath(path: string): Promise<Executables> {
    const paths = path.split(':')
    const promises = paths.map(x => findExecutablesInPath(x))
    return Promise.all(promises)
      .then(ArrayUtil.flatten)
      .then(ArrayUtil.uniq)
      .then(executables => new Executables(executables))
  }

  private executables: Set<string>

  private constructor(executables: string[]) {
    this.executables = new Set(executables)
  }

  /**
   * Find all programs in your PATH
   */
  public list(): string[] {
    return Array.from(this.executables.values())
  }

  /**
   * Check if the the given {{executable}} exists on the PATH
   */
  public isExecutableOnPATH(executable: string): boolean {
    return this.executables.has(executable)
  }

  /**
   * Look up documentation for the given executable.
   *
   * For now it simply tries to look up the MAN documentation.
   */
  public documentation(executable: string): Promise<string> {
    return ShUtil.execShellScript(`man ${executable} | col -b`).then(doc => {
      return !doc
        ? Promise.resolve(`No MAN page for ${executable}`)
        : Promise.resolve(doc)
    })
  }
}

/**
 * Only returns direct children, or the path itself if it's an executable.
 */
async function findExecutablesInPath(path: string): Promise<string[]> {
  path = FsUtil.untildify(path)

  try {
    const pathStats = await lstatAsync(path)

    if (pathStats.isDirectory()) {
      const childrenPaths = await readdirAsync(path)

      const files = []

      for (const childrenPath of childrenPaths) {
        try {
          const stats = await lstatAsync(join(path, childrenPath))
          if (isExecutableFile(stats)) {
            files.push(basename(childrenPath))
          }
        } catch (error) {
          // Ignore error
        }
      }

      return files
    } else if (isExecutableFile(pathStats)) {
      return [basename(path)]
    }
  } catch (error) {
    // Ignore error
  }

  return []
}

function isExecutableFile(stats: fs.Stats): boolean {
  const isExecutable = !!(1 & parseInt((stats.mode & parseInt('777', 8)).toString(8)[0]))
  return stats.isFile() && isExecutable
}
